Module if_exists

Module if_exists 

Zero-erasure if_exists/if_not_exists uni-constraint.

Filters A entities based on whether a matching B entity exists in another collection. The result is still a uni-constraint over A, not a bi-constraint.

Structs

IfExistsUniConstraint
Zero-erasure uni-constraint with existence filtering.

Enums

ExistenceMode
Whether to include A entities that have or don’t have matching B entities.