Module director

Module director 

Score director implementations.

The score director manages solution state and score calculation.

Score Director Types

Re-exports

pub use recording::RecordingScoreDirector;
pub use shadow_aware::ShadowAwareScoreDirector;
pub use shadow_aware::ShadowVariableSupport;
pub use shadow_aware::SolvableSolution;

Modules

recording
Recording score director for automatic undo tracking.
shadow_aware
Shadow-aware score director for solutions with shadow variables.
typed
Typed score director for zero-erasure incremental scoring.

Structs

ScoreDirectorFactory
Factory for creating score directors (zero-erasure).
SimpleScoreDirector
A simple score director that recalculates the full score each time (zero-erasure).

Traits

ScoreDirector
The score director manages solution state and score calculation.