Module stream

Module stream 

Re-exports

pub use collection_extract::CollectionExtract;
pub use collection_extract::FlattenExtract;
pub use collection_extract::VecExtract;
pub use join_target::JoinTarget;
pub use key_extract::EntityKeyAdapter;
pub use key_extract::KeyExtract;

Modules

collection_extract
collector
filter
join_target
joiner
key_extract

Structs

BalanceConstraintBuilder
BalanceConstraintStream
BiConstraintBuilder
BiConstraintStream
ComplementedConstraintBuilder
ComplementedConstraintStream
ConstraintFactory
CrossBiConstraintBuilder
CrossBiConstraintStream
CrossComplementedGroupedConstraintBuilder
CrossComplementedGroupedConstraintStream
CrossGroupedConstraintBuilder
CrossGroupedConstraintStream
ExistsConstraintBuilder
ExistsConstraintStream
FixedWeight
Public wrapper for fixed score weights that are not one of the built-in score types.
FlattenedBiConstraintBuilder
FlattenedBiConstraintStream
FlattenedCollectionTarget
GroupedConstraintBuilder
GroupedConstraintStream
HardWeight
Public wrapper that marks a fixed or dynamic weight as hard metadata.
JoinedProjectedSource
PentaConstraintBuilder
PentaConstraintStream
ProjectedBiConstraintBuilder
ProjectedBiConstraintStream
ProjectedComplementedGroupedConstraintBuilder
ProjectedComplementedGroupedConstraintStream
ProjectedConstraintBuilder
ProjectedConstraintStream
ProjectedGroupedConstraintBuilder
ProjectedGroupedConstraintStream
QuadConstraintBuilder
QuadConstraintStream
TriConstraintBuilder
TriConstraintStream
UniConstraintBuilder
UniConstraintStream

Enums

ExistenceMode

Traits

ExistenceTarget
Projection
ProjectionSink

Functions

fixed_weight
Wraps a fixed score weight while preserving static dispatch.
hard_weight
Wraps a weight and forces hard constraint metadata while preserving static dispatch.