Score director implementations.
The score director manages solution state and score calculation.
Score Director Types
SimpleScoreDirector- Full recalculation (baseline)- [
TypedScoreDirector] - Zero-erasure incremental scoring RecordingScoreDirector- Automatic undo tracking wrapperShadowAwareScoreDirector- Shadow variable integration wrapper
Re-exports
pub use recording::RecordingScoreDirector;pub use shadow_aware::ShadowAwareScoreDirector;pub use shadow_aware::ShadowVariableSupport;pub use shadow_aware::SolvableSolution;
Modules
- recording
- Recording score director for automatic undo tracking.
- shadow_
aware - Shadow-aware score director for solutions with shadow variables.
- typed
- Typed score director for zero-erasure incremental scoring.
Structs
- Score
Director Factory - Factory for creating score directors (zero-erasure).
- Simple
Score Director - A simple score director that recalculates the full score each time (zero-erasure).
Traits
- Score
Director - The score director manages solution state and score calculation.